@@ -1,11 +1,25 @@
-import json as jsonlib
 import functools
 import webtest
 
 from urllib.parse import urlencode
 
-from cytra import Application
+from cytra import Application, json as jsonlib
 from cytra.db import DatabaseManager, metadata
+from cytra.helpers import NestedNamespace
+
+
+class WebtestResponse(webtest.TestResponse):
+    @property
+    def json(self):
+        return NestedNamespace(super().json)
+
+
+class WebtestRequest(webtest.TestRequest):
+    ResponseClass = WebtestResponse
+
+
+class WebtestApp(webtest.TestApp):
+    RequestClass = WebtestRequest
 
 
 class TestingApp:
@@ -16,9 +30,12 @@ class TestingApp:
         app.configure(config_filename)
         if app.config.get("sqlalchemy") and app.config.get("sqlalchemy_test"):
             app.config.sqlalchemy = app.config.sqlalchemy_test
+            self._db_enabled = True
+        else:
+            self._db_enabled = False
 
     def __enter__(self):
-        if self.app.config.get("sqlalchemy"):
+        if self._db_enabled:
             with DatabaseManager(
                 engine_kwargs=self.app.config.sqlalchemy.engine,
                 admin_engine_kwargs=self.app.config.sqlalchemy_admin.engine,
@@ -28,8 +45,13 @@ class TestingApp:
                 dbm.create_database()
 
         self.app.setup()
-        self.db = self.app.db.session_factory()
-        self.testapp = webtest.TestApp(self.app)
+
+        if self._db_enabled:
+            self.db = self.app.db.session_factory()
+            metadata.drop_all(bind=self.db.get_bind())
+            metadata.create_all(bind=self.db.get_bind())
+
+        self.testapp = WebtestApp(self.app, json_encoder=jsonlib)
         self.testapp.extra_environ["HTTP_ACCEPT"] = "application/json"
         self.testapp.extra_environ["REMOTE_ADDR"] = "127.0.0.1"
         self.testapp.extra_environ["HTTP_APP_CLIENT"] = "MapleWeb-1.2.3-beta"
@@ -38,12 +60,12 @@ class TestingApp:
             "AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 "
             "Mobile/9B179 Safari/7534.48.3; fa-IR; some; extra; info)"
         )
-        metadata.drop_all(bind=self.db.get_bind())
-        metadata.create_all(bind=self.db.get_bind())
         return self
 
     def __exit__(self, *args):
-        self.db.close()
+        if self._db_enabled:
+            self.db.close()
+
         self.app.shutdown()
 
     def get_access_token(self, as_):  # pragma: nocover
